defmodule DecentralisedBookIndex.SyncTasks.SyncPublishersTask do
  alias DecentralisedBookIndex.Sync.ApiClients.FetchJsons
  alias DecentralisedBookIndex.Sync.DataTransformers.PublisherTransformer
  alias DecentralisedBookIndex.Sync.PublisherSync

  alias DecentralisedBookIndex.Metadata.DBIServer

  require Logger

  def sync(%DBIServer{} = server, url_params \\ "") do
    url = "#{server.url}/api/v1/json/publishers#{url_params}"
    FetchJsons.get(url, sync_closure(server))

    server
  end

  def sync_chunk(json_chunk, server_id) do
    for json <- json_chunk do
      with {:ok, attrs} <- PublisherTransformer.from_json(json),
           :ok <- PublisherSync.create_update(attrs, server_id) do
        :ok
      else
        {:error, reason} ->
          Logger.error("Pipeline error: #{inspect(reason)}")
      end
    end

    []
  end

  def sync_closure(server) do
    fn json_chunk ->
      sync_chunk(json_chunk, server.id)
    end
  end
end
