defmodule DecentralisedBookIndexWeb.BookLive.FormComponent do
  use DecentralisedBookIndexWeb, :live_component

  alias DecentralisedBookIndex.Metadata

  @impl true
  def render(assigns) do
    ~H"""
    <div>
      <.header>
        {@title}
      </.header>

      <.simple_form
        for={@form}
        id="book-form"
        phx-target={@myself}
        phx-change="validate"
        phx-submit="save"
      >
        <%= if @form.source.type == :create do %>
          <.input field={@form[:title]} type="text" label="Title" />
          <.input field={@form[:cover_image_url]} type="text" label="Cover image url" />
          <.input
            field={@form[:description]}
            type="textarea"
            label="Description"
          />
          <.bids_inputs form={@form} myself={@myself} />
          <.author_roles_inputs form={@form} myself={@myself} notify_component={@myself} />
          <.input field={@form[:format]} type="text" label="Format" />
          <.input
            field={@form[:language]}
            type="text"
            label="Language"
          />
          <.input field={@form[:page_count]} type="number" label="Page count" />
          <div>
            <.input field={@form[:publisher_id]} type="text" label="Publisher" type="hidden" show_errors?={false} />
            <.selected_publisher publisher_form={@form[:publisher_id]} notify_component={@myself} />
          </div>
          <.input field={@form[:published]} type="date" label="Published" />
          <div>
            <.input field={@form[:book_editions_registry_id]} type="hidden" label="Book Editions" />
            <.selected_book_edition book_editions_registry_form={@form[:book_editions_registry_id]} notify_component={@myself} />
          </div>
        <% end %>
        <%= if @form.source.type == :update do %>
          <.input field={@form[:title]} type="text" label="Title" />
          <.input field={@form[:cover_image_url]} type="text" label="Cover image url" />
          <.input
            field={@form[:description]}
            type="textarea"
            label="Description"
          />
          <.bids_inputs form={@form} myself={@myself} />
          <.author_roles_inputs form={@form} myself={@myself} notify_component={@myself} />
          <.input field={@form[:format]} type="text" label="Format" />
          <.input
            field={@form[:language]}
            type="text"
            label="Language"
          />
          <.input field={@form[:page_count]} type="number" label="Page count" />
          <div>
            <.input field={@form[:publisher_id]} type="text" label="Publisher" type="hidden" show_errors?={false} />
            <.selected_publisher publisher_form={@form[:publisher_id]} notify_component={@myself} />
          </div>
          <.input field={@form[:published]} type="date" label="Published" />
          <div>
            <.input field={@form[:book_editions_registry_id]} type="hidden" label="Book Editions" />
            <.selected_book_edition book_editions_registry_form={@form[:book_editions_registry_id]} notify_component={@myself} />
          </div>
        <% end %>

        <:actions>
          <.save_button phx-disable-with="Saving...">
            Save
          </.save_button>
          <.cancel_button phx-click="cancel" phx-target={@myself}>
            Cancel
          </.cancel_button>
        </:actions>
      </.simple_form>

      <.modal
        :if={@select_publisher_open? == true}
        id="select-publisher-modal"
        show
        on_cancel={JS.push("close-select-publisher", target: @myself)}
      >
        <.live_component
          id="select-publisher"
          module={SelectPublisher}
          current_user={@current_user}
          notify_component={@myself}
        />
      </.modal>

      <.modal
        :if={@select_book_edition_open? == true}
        id="select-book-edition-modal"
        show
        on_cancel={JS.push("close-select-book-edition", target: @myself)}
      >
        <.live_component
          id="select-book-edition"
          module={SelectBookEdition}
          current_user={@current_user}
          notify_component={@myself}
        />
      </.modal>

      <.modal
        :if={@select_author_open? == true}
        id="select-author-modal"
        show
        on_cancel={JS.push("close-select-author", target: @myself)}
      >
        <.live_component
          id="select-author"
          module={SelectAuthor}
          current_user={@current_user}
          notify_component={@myself}
          form_path={@form_path}
        />
      </.modal>
    </div>
    """
  end

  def bids_inputs(assigns) do
    ~H"""
    <h2 class="font-semibold leading-8 text-zinc-800 dark:text-white">Book ids</h2>

    <table class="w-full">
      <thead class="border-b border-zinc-100">
        <tr>
          <th class="text-left font-medium text-sm pb-1 px-3 w-10"></th>
          <th class="text-left font-medium text-sm pb-1 px-3 w-36 dark:text-white">Type</th>
          <th class="text-left font-medium text-sm pb-1 px-3 dark:text-white">Id</th>
          <th class="text-left font-medium text-sm pb-1 px-3 w-12"></th>
        </tr>
      </thead>
      <tbody phx-hook="bidSort" id="bidSort" phx-target={@myself}>
        <.inputs_for :let={bid_form} field={@form[:bids]}>
          <tr data-id={bid_form.index}>
            <td class="px-3 w-10 pt-2">
              <svg class="w-6 h-6 text-gray-800 dark:text-white handle cursor-pointer" aria-hidden="true" xmlns="http://www.w3.org/2000/svg" width="24" height="24" fill="none" viewBox="0 0 24 24">
                <path stroke="currentColor" stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="m8 15 4 4 4-4m0-6-4-4-4 4"/>
              </svg>
            </td>
            <td class="px-3 w-36">
              <label for={bid_form[:type].id} class="hidden">Type</label>
              <.input field={bid_form[:type]} />
            </td>
            <td class="px-3">
              <label for={bid_form[:bid].id} class="hidden">Id</label>
              <.input field={bid_form[:bid]} />
            </td>
            <td class="flex items-center justify-center w-12 pt-2">
              <.button_link
                phx-click="remove-bid"
                phx-value-path={bid_form.name}
                phx-target={@myself}
                kind="error"
                size="xs"
                inverse
              >
                <svg class="w-6 h-6 text-gray-800 dark:text-white" aria-hidden="true" xmlns="http://www.w3.org/2000/svg" width="24" height="24" fill="currentColor" viewBox="0 0 24 24">
                  <path fill-rule="evenodd" d="M8.586 2.586A2 2 0 0 1 10 2h4a2 2 0 0 1 2 2v2h3a1 1 0 1 1 0 2v12a2 2 0 0 1-2 2H7a2 2 0 0 1-2-2V8a1 1 0 0 1 0-2h3V4a2 2 0 0 1 .586-1.414ZM10 6h4V4h-4v2Zm1 4a1 1 0 1 0-2 0v8a1 1 0 1 0 2 0v-8Zm4 0a1 1 0 1 0-2 0v8a1 1 0 1 0 2 0v-8Z" clip-rule="evenodd"/>
                </svg>
              </.button_link>
            </td>
          </tr>
        </.inputs_for>
      </tbody>
    </table>

    <.add_button_link phx-click="add-bid" phx-target={@myself} class="w-44">
      Add Book Id
    </.add_button_link>
    """
  end

  @impl true
  def handle_event("add-bid", _params, socket) do
    socket =
      update(socket, :form, fn form ->
        AshPhoenix.Form.add_form(form, :bids)
      end)

    {:noreply, socket}
  end

  @impl true
  def handle_event("remove-bid", %{"path" => path}, socket) do
    socket =
      update(socket, :form, fn form ->
        AshPhoenix.Form.remove_form(form, path)
      end)

    {:noreply, socket}
  end

  @impl true
  def handle_event("reorder-bids", %{"order" => order}, socket) do
    socket =
      update(socket, :form, fn form ->
        AshPhoenix.Form.sort_forms(form, [:bids], order)
      end)

    {:noreply, socket}
  end

  def author_roles_inputs(assigns) do
    ~H"""
    <h2 class="font-semibold leading-8 text-zinc-800 dark:text-white">Authors</h2>

    <table class="w-full">
      <thead class="border-b border-zinc-100">
        <tr>
          <th class="text-left font-medium text-sm pb-1 px-3 w-10"></th>
          <th class="text-left font-medium text-sm pb-1 px-3 w-48 dark:text-white">Author</th>
          <th class="text-left font-medium text-sm pb-1 px-3 dark:text-white">Role</th>
          <th class="text-left font-medium text-sm pb-1 px-3 w-12"></th>
        </tr>
      </thead>
      <tbody phx-hook="authorRoleSort" id="authorRoleSort" phx-target={@myself}>
        <.inputs_for :let={author_roles_form} field={@form[:author_roles]}>
          <tr data-id={author_roles_form.index}>
            <td class="px-3 w-10 pt-2">
              <svg class="w-6 h-6 text-gray-800 dark:text-white handle cursor-pointer" aria-hidden="true" xmlns="http://www.w3.org/2000/svg" width="24" height="24" fill="none" viewBox="0 0 24 24">
                <path stroke="currentColor" stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="m8 15 4 4 4-4m0-6-4-4-4 4"/>
              </svg>
            </td>
            <td class="px-3 w-48">
              <label for={author_roles_form[:author_id].id} class="hidden">Type</label>
              <.input field={author_roles_form[:author_id]} type="hidden" show_errors?={false} />
              <.selected_author
                author_form={author_roles_form[:author_id]}
                notify_component={@notify_component}
                form_path={author_roles_form.name}
                class="mt-2 w-full"
              />
            </td>
            <td class="px-3">
              <label for={author_roles_form[:role].id} class="hidden">Id</label>
              <.input field={author_roles_form[:role]} />
            </td>
            <td class="flex items-center justify-center w-12 pt-2">
              <.button_link
                phx-click="remove-author-role"
                phx-value-path={author_roles_form.name}
                phx-target={@myself}
                kind="error"
                size="xs"
                inverse
              >
                <svg class="w-6 h-6 text-gray-800 dark:text-white" aria-hidden="true" xmlns="http://www.w3.org/2000/svg" width="24" height="24" fill="currentColor" viewBox="0 0 24 24">
                  <path fill-rule="evenodd" d="M8.586 2.586A2 2 0 0 1 10 2h4a2 2 0 0 1 2 2v2h3a1 1 0 1 1 0 2v12a2 2 0 0 1-2 2H7a2 2 0 0 1-2-2V8a1 1 0 0 1 0-2h3V4a2 2 0 0 1 .586-1.414ZM10 6h4V4h-4v2Zm1 4a1 1 0 1 0-2 0v8a1 1 0 1 0 2 0v-8Zm4 0a1 1 0 1 0-2 0v8a1 1 0 1 0 2 0v-8Z" clip-rule="evenodd"/>
                </svg>
              </.button_link>
            </td>
          </tr>
        </.inputs_for>
      </tbody>
    </table>

    <.add_button_link phx-click="add-author-role" phx-target={@myself} class="w-40">
      Add Author
    </.add_button_link>
    """
  end

  @impl true
  def handle_event("add-author-role", _params, socket) do
    socket =
      update(socket, :form, fn form ->
        AshPhoenix.Form.add_form(form, :author_roles)
      end)

    {:noreply, socket}
  end

  @impl true
  def handle_event("remove-author-role", %{"path" => path}, socket) do
    socket =
      update(socket, :form, fn form ->
        AshPhoenix.Form.remove_form(form, path)
      end)

    {:noreply, socket}
  end

  @impl true
  def handle_event("reorder-author-role", %{"order" => order}, socket) do
    socket =
      update(socket, :form, fn form ->
        AshPhoenix.Form.sort_forms(form, [:author_roles], order)
      end)

    {:noreply, socket}
  end

  @impl true
  def update(assigns, socket) do
    {:ok,
     socket
     |> assign(assigns)
     |> assign(:select_publisher_open?, false)
     |> assign(:select_author_open?, false)
     |> assign(:select_book_edition_open?, false)
     |> assign(:form_path, nil)
     |> assign_form()}
  end

  @impl true
  def handle_event("validate", %{"book" => book_params}, socket) do
    {:noreply, assign(socket, form: AshPhoenix.Form.validate(socket.assigns.form, book_params))}
  end

  def handle_event("save", %{"book" => book_params}, socket) do
    case AshPhoenix.Form.submit(socket.assigns.form, params: book_params) do
      {:ok, book} ->
        notify_parent({:saved, book})

        socket =
          socket
          |> put_flash(:info, "Book #{socket.assigns.form.source.type}d successfully")
          |> redirect(to: patch_url(socket.assigns.action, book.id))

        {:noreply, socket}

      {:error, form} ->
        {:noreply, assign(socket, form: form)}
    end
  end

  def handle_event("cancel", _params, socket) do
    book = socket.assigns.book
    book_id = if book == nil, do: nil, else: book.id

    socket =
      socket
      |> push_navigate(to: patch_url(socket.assigns.action, book_id))

    {:noreply, socket}
  end

  defp notify_parent(msg), do: send(self(), {__MODULE__, msg})

  defp assign_form(%{assigns: %{book: book, alternative_book_id: alternative_book_id}} = socket) do
    alternative_edition_id =
      if alternative_book_id == nil do
        nil
      else
        alternative_book = Metadata.get_book_by_id!(alternative_book_id)
        alternative_book.book_editions_registry_id
      end

    form =
      if book do
        book =
          book
          |> Ash.load!([:bids, :author_roles, :publisher])

        AshPhoenix.Form.for_update(book, :update, as: "book", actor: socket.assigns.current_user)
      else
        form =
          AshPhoenix.Form.for_create(DecentralisedBookIndex.Metadata.Book, :create,
            as: "book",
            actor: socket.assigns.current_user
          )
          |> set_book_edition_registry_id(alternative_edition_id)
      end

    assign(socket, form: to_form(form))
  end

  defp set_book_edition_registry_id(form, alternative_edition_id) do
    if alternative_edition_id != nil do
      form
      |> AshPhoenix.Form.update_params(
        &Map.put(&1, "book_editions_registry_id", alternative_edition_id)
      )
    else
      form
    end
  end

  defp patch_url(action, book_id) do
    case action do
      :edit -> ~p"/books/#{book_id}"
      :new -> ~p"/books"
      _ -> ~p"/books"
    end
  end

  def handle_event("select-publisher", %{"publisher" => publisher_id}, socket) do
    socket =
      socket
      |> update(:form, fn form ->
        form
        |> AshPhoenix.Form.update_params(&Map.put(&1, "publisher_id", publisher_id))
      end)
      |> assign(:select_publisher_open?, false)

    {:noreply, socket}
  end

  def handle_event("open-select-publisher", _params, socket) do
    socket =
      socket
      |> assign(:select_publisher_open?, true)

    {:noreply, socket}
  end

  def handle_event("close-select-publisher", _params, socket) do
    socket =
      socket
      |> assign(:select_publisher_open?, false)

    {:noreply, socket}
  end

  def handle_event("select-author", %{"author" => author_id, "path" => path}, socket) do
    socket =
      socket
      |> update(:form, fn form ->
        form
        |> AshPhoenix.Form.update_form(path, fn nested_form ->
          params = Map.put(nested_form.params, "author_id", author_id)
          AshPhoenix.Form.validate(nested_form, params)
        end)
      end)
      |> assign(:select_author_open?, false)

    {:noreply, socket}
  end

  def handle_event("open-select-author", %{"path" => path}, socket) do
    socket =
      socket
      |> assign(:select_author_open?, true)
      |> assign(:form_path, path)

    {:noreply, socket}
  end

  def handle_event("close-select-author", _params, socket) do
    socket =
      socket
      |> assign(:select_author_open?, false)
      |> assign(:form_path, nil)

    {:noreply, socket}
  end

  def handle_event("select-book-edition", %{"book-edition" => book_edition_id}, socket) do
    socket =
      socket
      |> update(:form, fn form ->
        form
        |> AshPhoenix.Form.update_params(&Map.put(&1, "book_editions_registry_id", book_edition_id))
      end)
      |> assign(:select_book_edition_open?, false)

    {:noreply, socket}
  end

  def handle_event("open-select-book-edition", _params, socket) do
    socket =
      socket
      |> assign(:select_book_edition_open?, true)

    {:noreply, socket}
  end

  def handle_event("close-select-book-edition", _params, socket) do
    socket =
      socket
      |> assign(:select_book_edition_open?, false)

    {:noreply, socket}
  end
end
